@@ -11,7 +11,9 @@ import logging
 _logger = logging.getLogger("Appearance")
 ###############
 
+import xml
 from xml.dom import minidom
+import sys
 import os
 import re
 import colorsys
@@ -19,7 +21,7 @@ from math import log
 
 from Onboard             import Exceptions
 from Onboard.utils       import hexstring_to_float, brighten, toprettyxml, \
-                                TreeItem, Version
+                                TreeItem, Version, unicode_str
 
 import Onboard.utils as utils
 
@@ -39,20 +41,26 @@ class Theme:
     # changed range of key_gradient_direction
     THEME_FORMAT_1_1 = Version(1, 1)
 
-    THEME_FORMAT = THEME_FORMAT_1_1
+    # onboard 0.98, added shadow keys
+    THEME_FORMAT_1_2 = Version(1, 2)
+
+    THEME_FORMAT = THEME_FORMAT_1_2
 
     # core theme members
     # name, type, default
     attributes = [
             ["color_scheme_basename", "s", ""],
+            ["background_gradient", "d", 0.0],
             ["key_style", "s", "flat"],
-            ["roundrect_radius", "f", 0],
-            ["key_size", "f", 100],
-            ["key_fill_gradient", "f", 0],
-            ["key_stroke_gradient", "f", 0],
-            ["key_gradient_direction", "f", 0],
+            ["roundrect_radius", "d", 0.0],
+            ["key_size", "d", 100.0],
+            ["key_fill_gradient", "d", 0.0],
+            ["key_stroke_gradient", "d", 0.0],
+            ["key_gradient_direction", "d", 0.0],
             ["key_label_font", "s", ""],
-            ["key_label_overrides", "d", {}]   # dict {name:(key:group)}
+            ["key_label_overrides", "a{s[ss]}", {}],   # dict {name:(key:group)}
+            ["key_shadow_strength", "d", 0.0],
+            ["key_shadow_size", "d", 0.0],
             ]
 
     def __init__(self):
@@ -91,8 +99,8 @@ class Theme:
         """ Applies the theme to config properties/gsettings. """
         filename = self.get_color_scheme_filename()
         if not filename:
-            _logger.error(_("Color scheme for theme '{filename}' not found") \
-                            .format(filename=self.filename))
+            _logger.error(_format("Color scheme for theme '{filename}' not found", \
+                                  filename=self.filename))
             return False
 
         config.theme_settings.set_color_scheme_filename(filename, save)
@@ -219,7 +227,8 @@ class Theme:
         filenames = Theme.find_themes(path)
         for filename in filenames:
             theme = Theme.load(filename, is_system)
-            themes.append(theme)
+            if theme:
+                themes.append(theme)
         return themes
 
     @staticmethod
@@ -240,7 +249,7 @@ class Theme:
 
         result = None
 
-        _file = open(filename)
+        _file = open(filename, encoding = "UTF-8")
         try:
             domdoc = minidom.parse(_file).documentElement
             try:
@@ -280,8 +289,10 @@ class Theme:
 
                             if _type == "i":
                                 value = int(value)
-                            if _type == "f":
+                            if _type == "d":
                                 value = float(value)
+                            if _type == "ad":
+                                value = [float(s) for s in value.split(",")]
 
                             # upgrade to current file format
                             if format < Theme.THEME_FORMAT_1_1:
@@ -298,13 +309,17 @@ class Theme:
                 theme.is_system = is_system
                 theme.system_exists = is_system
                 result = theme
-            except Exceptions.ThemeFileError as xxx_todo_changeme:
-                (ex) = xxx_todo_changeme
-                raise Exceptions.ThemeFileError(_("Error loading ")
-                    + filename, chained_exception = ex)
             finally:
                 domdoc.unlink()
 
+        except (Exceptions.ThemeFileError,
+                xml.parsers.expat.ExpatError) as ex:
+            _logger.error(_format("Error loading theme '{filename}'. "
+                                  "{exception}: {cause}",
+                                  filename = filename,
+                                  exception = type(ex).__name__,
+                                  cause = unicode_str(ex)))
+            result = None
         finally:
             _file.close()
 
@@ -349,8 +364,10 @@ class Theme:
                         pass
                     elif _type == "i":
                         value = str(value)
-                    elif _type == "f":
-                        value = str(round(value, 2))
+                    elif _type == "d":
+                        value = str(round(float(value), 2))
+                    elif _type == "ad":
+                        value = ", ".join(str(d) for d in value)
                     else:
                         assert(False) # attribute of unknown type
 
@@ -361,8 +378,11 @@ class Theme:
 
             pretty_xml = toprettyxml(domdoc)
 
-            with open(self.filename, "w") as _file:
-                _file.write(pretty_xml.encode("UTF-8"))
+            with open(self.filename, "w", encoding = "UTF-8") as _file:
+                if sys.version_info.major >= 3:
+                    _file.write(pretty_xml)
+                else:
+                    _file.write(pretty_xml.encode("UTF-8"))
 
         except Exception as xxx_todo_changeme2:
             (ex) = xxx_todo_changeme2
@@ -379,7 +399,7 @@ class ColorScheme(object):
     Any color definition may be omitted. Undefined colors fall back
     to color scheme defaults first, then to hard coded default colors.
     """
-    
+
     # onboard 0.95
     COLOR_SCHEME_FORMAT_LEGACY = Version(1, 0)
 
@@ -408,7 +428,11 @@ class ColorScheme(object):
         return False
 
     def get_key_rgba(self, key, element, state = None):
-        # build a dict of supported key states
+        """
+        Get the color for the given key element and optionally key state.
+        If state is None, the key state is taken from the key itself.
+        """
+
         if state is None:
             state = {}
             state["prelight"]    =  key.prelight
@@ -424,23 +448,23 @@ class ColorScheme(object):
         root_opacity = None
         key_group = None
 
-        # first try to find the theme_id then fall back to the regular id
+        # first try to find the theme_id then fall back to the generic id
         for id in [key.theme_id, key.id]:
             key_group = self.root.find_key_id(id)
             if key_group:
                 rgb, opacity = key_group.find_element_color(element, state)
                 break
 
-        # Get root colors in case key id wasn't mentioned
-        # anywhere in the color scheme.
+        # Get root colors as fallback for the case when key id
+        # wasn't mentioned anywhere in the color scheme.
         root_key_group = self.root.get_default_key_group()
         if root_key_group:
             root_rgb, root_opacity = \
                     root_key_group.find_element_color(element, state)
 
         # Special case for layer buttons:
-        # don't get fill color from root group,
-        # we want the layer fill color instead.
+        # don't take fill color from the root group,
+        # we want the layer fill color instead (via get_key_default_rgba()).
         if element == "fill" and key.is_layer_button():
             # Don't pick layer fill opacity when there is
             # an rgb color defined in the color scheme.
@@ -450,6 +474,7 @@ class ColorScheme(object):
                 if opacity is None:
                     opacity = 1.0
         elif key_group is None:
+            # All other colors fall back to the root groups colors
             rgb = root_rgb
             opacity = root_opacity
 
@@ -484,13 +509,13 @@ class ColorScheme(object):
                 # default color is layer fill color (as in onboard <=0.95).
                 layer_index = key.get_layer_index()
                 rgba = self.get_layer_fill_rgba(layer_index)
-                
+
             elif state.get("pressed"):
                 new_state = dict(list(state.items()))
                 new_state["pressed"] = False
                 rgba = self.get_key_rgba(key, element, new_state)
 
-                # Make the default pressed color a slightly darker 
+                # Make the default pressed color a slightly darker
                 # or brighter variation of the unpressed color.
                 h, l, s = colorsys.rgb_to_hls(*rgba[:3])
 
@@ -505,6 +530,24 @@ class ColorScheme(object):
 
             elif state.get("scanned"):
                 rgba = colors["scanned"]
+                # Make scanned active modifier keys stick out by mixing
+                # scanned color with non-scanned color.
+                if state.get("active"): # includes locked
+                    # inactive scanned color
+                    new_state = dict(list(state.items()))
+                    new_state["active"] = False
+                    new_state["locked"] = False
+                    scanned = self.get_key_rgba(key, element, new_state)
+
+                    # unscanned fill color
+                    new_state = dict(list(state.items()))
+                    new_state["scanned"] = False
+                    fill = self.get_key_rgba(key, element, new_state)
+
+                    # mix inactive scanned color with unscanned fill color
+                    for i in range(4):
+                        rgba[i] = (scanned[i] + fill[i]) / 2.0
+
             elif state.get("prelight"):
                 rgba = colors["prelight"]
             elif state.get("locked"):
@@ -669,7 +712,7 @@ class ColorScheme(object):
 
         color_scheme = None
 
-        f = open(filename)
+        f = open(filename, encoding="UTF-8")
         try:
             dom = minidom.parse(f).documentElement
             name = dom.attributes["name"].value
@@ -682,13 +725,13 @@ class ColorScheme(object):
             if format >= ColorScheme.COLOR_SCHEME_FORMAT_TREE:   # tree format?
                 items = ColorScheme._parse_dom_node(dom, None, {})
             else:
-                _logger.warning( \
-                  _("Loading legacy color scheme format '{old_format}', "
-                    "please consider upgrading to current format " \
-                    "'{new_format}': '{filename}'") \
-                .format(old_format = format, 
-                        new_format = ColorScheme.COLOR_SCHEME_FORMAT,
-                        filename = filename))
+                _logger.warning(_format( \
+                    "Loading legacy color scheme format '{old_format}', "
+                    "please consider upgrading to current format "
+                    "'{new_format}': '{filename}'",
+                    old_format = format,
+                    new_format = ColorScheme.COLOR_SCHEME_FORMAT,
+                    filename = filename))
 
                 items = ColorScheme._parse_legacy_color_scheme(dom)
 
@@ -702,6 +745,12 @@ class ColorScheme(object):
                 color_scheme.is_system = is_system
                 color_scheme.root = root
                 #print(root.dumps())
+        except xml.parsers.expat.ExpatError as ex:
+            _logger.error(_format("Error loading color scheme '{filename}'. "
+                                  "{exception}: {cause}",
+                                  filename = filename,
+                                  exception = type(ex).__name__,
+                                  cause = unicode_str(ex)))
         finally:
             f.close()
 
@@ -762,10 +811,11 @@ class ColorScheme(object):
         # check for duplicate key definitions
         for key_id in ids:
             if key_id in used_keys:
-                raise ValueError(_("Duplicate key_id '{}' found "
-                  "in color scheme file. "
-                  "Key_ids must occur only once."
-                 .format(key_id)))
+                raise ValueError(_format("Duplicate key_id '{}' found "
+                                         "in color scheme file. "
+                                         "Key_ids must occur only once.",
+                                         key_id))
+
         used_keys.update(list(zip(ids, ids)))
 
         item.key_ids = ids
@@ -893,10 +943,10 @@ class ColorScheme(object):
             # check for duplicate key definitions
             for key_id in key_ids:
                 if key_id in used_keys:
-                    raise ValueError(_("Duplicate key_id '{}' found "
-                      "in color scheme file. "
-                      "Key_ids must occur only once."
-                     .format(key_id)))
+                    raise ValueError(_format("Duplicate key_id '{}' found "
+                                             "in color scheme file. "
+                                             "Key_ids must occur only once.",
+                                             key_id))
             used_keys.update(list(zip(key_ids, key_ids)))
 
             colors = []
@@ -1004,7 +1054,7 @@ class Root(ColorSchemeItem):
     """ Container for a layers colors """
 
     def get_layers(self):
-        """ 
+        """
         Get list of layer items in order of appearance
         in the color scheme file.
         """
@@ -1015,8 +1065,8 @@ class Root(ColorSchemeItem):
         return layers
 
     def get_icons(self):
-        """ 
-        Get list of the icon items in order of appearance 
+        """
+        Get list of the icon items in order of appearance
         in the color scheme file.
         """
         icons = []
@@ -1070,7 +1120,7 @@ class Color(ColorSchemeItem):
 
 
 class KeyColor(Color):
-    """ 
+    """
     A single key (or layer) color.
     """
     state = None   # dict whith "pressed"=True, "active"=False, etc.
