@@ -115,29 +115,35 @@ class Keyboard:
 
 ##################
 
-    def __init__(self, vk):
+    def __init__(self):
+        self.layout = None
         self.scanner = None
-        self.vk = vk
-        self.unpress_timer = UnpressTimer(self)
+        self.vk = None
+        self.button_controllers = {}
+        self.canvas_rect = Rect()
+        
+        self._unpress_timer = UnpressTimer(self)
+        self._editing_snippet = False
 
-    def destruct(self):
-        self.cleanup()
-
-    def initial_update(self):
-        """ called when the layout has been loaded """
+        self.reset()
 
+    def reset(self):
         #List of keys which have been latched.
         #ie. pressed until next non sticky button is pressed.
         self._latched_sticky_keys = []
         self._locked_sticky_keys = []
 
-        self.canvas_rect = Rect()
+    def initial_update(self):
+        """ called when the layout has been loaded """
+        self.reset()
+
+        self._connect_button_controllers()
+        self.assure_valid_active_layer()
+        self.update_ui()
+
+    def _connect_button_controllers(self):
+        """ connect button controllers to button keys """
         self.button_controllers = {}
-        self._editing_snippet = False
-
-        self._last_canvas_extents = None
-
-        # connect button controllers to button keys
         types = [BCMiddleClick, BCSingleClick, BCSecondaryClick, BCDoubleClick, BCDragClick,
                  BCHoverClick,
                  BCHide, BCShowClick, BCMove, BCPreferences, BCQuit,
@@ -152,9 +158,6 @@ class Keyboard:
                     if type.id == key.id:
                         self.button_controllers[key] = type(self, key)
 
-        self.assure_valid_active_layer()
-        self.update_ui()
-
     def enable_scanner(self, enable):
         """ Config callback for scanner.enabled changes. """
         if enable:
@@ -238,12 +241,15 @@ class Keyboard:
     def _on_mods_changed(self):
         raise NotImplementedError()
 
+    def requires_delayed_press(self, key):
+        return key.id in ["MENU"]
+
     def press_key(self, key, button = 1, event_type = EventType.CLICK):
         if not key.sensitive:
             return
 
         # unpress the previous key
-        self.unpress_timer.reset()
+        self._unpress_timer.reset()
 
         key.pressed = True
 
@@ -252,7 +258,8 @@ class Keyboard:
                 self.alt_locked = True
                 self.vk.lock_mod(8)
 
-        if not key.sticky or not key.active:
+        if (not key.sticky or not key.active) and \
+           not self.requires_delayed_press(key):
             # press key
             self.send_press_key(key, button, event_type)
 
@@ -266,6 +273,18 @@ class Keyboard:
         if not key.sensitive:
             return
 
+#        if self.requires_delayed_press(key):
+#            Timer(0.2, self.do_release_key, key, button, event_type)
+#        else:
+        self._do_release_key(key, button, event_type)
+
+    def _do_release_key(self, key, button = 1, event_type = EventType.CLICK):
+        if not key.sensitive:
+            return
+
+        if self.requires_delayed_press(key):
+            self.send_press_key(key, button, event_type)
+
         # Was the key nothing but pressed before?
         extend_pressed_state = key.is_pressed_only()
 
@@ -301,7 +320,7 @@ class Keyboard:
         if extend_pressed_state and \
            not config.scanner.enabled:
             # Keep key pressed for a little longer for clear user feedback.
-            self.unpress_timer.start(key)
+            self._unpress_timer.start(key)
         else:
             # Unpress now to avoid flickering of the
             # pressed color after key release.
@@ -671,10 +690,10 @@ class Keyboard:
         return config.clickmapper
 
     def cleanup(self):
-        # resets still latched and locked modifier keys on exit
+        # reset still latched and locked modifier keys on exit
         self.release_latched_sticky_keys()
         self.release_locked_sticky_keys()
-        self.unpress_timer.stop()
+        self._unpress_timer.stop()
 
         for key in self.iter_keys():
             if key.pressed and key.action_type in \
@@ -685,8 +704,8 @@ class Keyboard:
 
                 # Release still pressed enter key when onboard gets killed
                 # on enter key press.
-                _logger.debug(_("Releasing still pressed key '{}'") \
-                             .format(key.id))
+                _logger.debug("Releasing still pressed key '{}'" \
+                              .format(key.id))
                 self.send_release_key(key)
 
         # Somehow keyboard objects don't get released
@@ -859,7 +878,7 @@ class BCHide(ButtonController):
     id = "hide"
 
     def release(self, button, event_type):
-        self.keyboard.set_user_visible(False)
+        self.keyboard.set_visible(False)
 
     def update(self):
         self.set_sensitive(not config.xid_mode) # insensitive in XEmbed mode
@@ -917,8 +936,8 @@ class BCMove(ButtonController):
         self.keyboard.stop_move_window()
 
     def update(self):
-        self.set_visible(not config.has_window_decoration())
-        self.set_sensitive(not config.xid_mode)
+        self.set_visible(not config.has_window_decoration() and \
+                         not config.xid_mode)
 
     def can_long_press(self):
         return not config.xid_mode
@@ -969,9 +988,9 @@ class BCPreferences(ButtonController):
         run_script("sokSettings")
 
     def update(self):
-        self.set_sensitive(not config.xid_mode and \
-                           not config.running_under_gdm and \
-                           not config.lockdown.disable_preferences)
+        self.set_visible(not config.xid_mode and \
+                         not config.running_under_gdm and \
+                         not config.lockdown.disable_preferences)
 
 
 class BCQuit(ButtonController):
@@ -982,6 +1001,6 @@ class BCQuit(ButtonController):
         self.keyboard.emit_quit_onboard()
 
     def update(self):
-        self.set_sensitive(not config.xid_mode and \
-                           not config.lockdown.disable_quit)
+        self.set_visible(not config.xid_mode and \
+                         not config.lockdown.disable_quit)
 
