@@ -11,23 +11,27 @@ import sys
 import time
 import traceback
 import signal
-import gettext
 import os.path
-import virtkey
-from gi.repository import GObject, Gio, Gdk, Gtk
 
-# setup gettext, install _() function for all modules
-app = "onboard"
-gettext.install(app, unicode=True)
+import dbus
+from dbus.mainloop.glib import DBusGMainLoop
+
+from gi.repository import GObject, Gio, Gdk, Gtk, GLib
+
+import virtkey
 
 from Onboard.Indicator import Indicator
 from Onboard.Keyboard import Keyboard
 from Onboard.Scanner import Scanner
 from Onboard.KeyGtk import *
 from Onboard.KbdWindow import KbdWindow, KbdPlugWindow
-from Onboard.KeyboardSVG import KeyboardSVG
-from Onboard.utils       import show_confirmation_dialog, CallOnce, timeit
-from Onboard.Appearance import Theme
+from Onboard.KeyboardGTK import KeyboardGTK
+from Onboard.LayoutLoaderSVG import LayoutLoaderSVG
+from Onboard.Appearance import Theme, ColorScheme
+from Onboard.IconPalette import IconPalette
+from Onboard.utils      import show_confirmation_dialog, CallOnce, Process, \
+                               unicode_str
+import Onboard.osk as osk
 
 ### Config Singleton ###
 from Onboard.Config import Config
@@ -35,8 +39,8 @@ config = Config()
 ########################
 
 import Onboard.KeyCommon
-import Onboard.utils as utils
 
+app = "onboard"
 DEFAULT_FONTSIZE = 10
 
 class OnboardGtk(Gtk.Application):
@@ -51,6 +55,30 @@ class OnboardGtk(Gtk.Application):
 
     def __init__(self):
 
+        # Make sure windows get "onboard", "Onboard" as name and class
+        # For some reason they aren't correctly set when onboard is started
+        # from outside the source directory (Precise).
+        GLib.set_prgname(str(app))
+        Gdk.set_program_class(app[0].upper() + app[1:])
+
+        # Use D-bus main loop by default
+        DBusGMainLoop(set_as_default=True)
+
+        # Onboard in Ubuntu on first start silently embeds itself into
+        # gnome-screen-saver and stays like this until embedding is manually
+        # turned off.
+        # If gnome's "Typing Assistent" is disabled, only show onboard in
+        # gss when there is already a non-embedded instance running in
+        # the user session (LP: 938302).
+        bus = dbus.SessionBus()
+        has_remote_instance = bus.name_has_owner(self.ONBOARD_APP_ID)
+        if config.xid_mode and \
+           not (config.gnome_a11y and \
+                config.gnome_a11y.screen_keyboard_enabled):
+            if Process.was_launched_by("gnome-screensaver") and \
+               not has_remote_instance:
+                sys.exit(0)
+
         if config.options.allow_multiple_instances or \
            config.xid_mode:
             app_flags = Gio.ApplicationFlags.NON_UNIQUE
@@ -60,33 +88,38 @@ class OnboardGtk(Gtk.Application):
         super(OnboardGtk, self).__init__(application_id=OnboardGtk.ONBOARD_APP_ID,
                                          flags=app_flags)
 
-        # exit on Ctrl+C
-        # This almost works, but still requires a motion event
-        # or somthing similar to actually quit.
-        sys.excepthook = self.excepthook
-
         _logger.info("Entering mainloop of onboard")
         self.run(None)
 
-    def excepthook(self, type, value, traceback):
-        """
-        Exit onboard on Ctrl+C press.
-        """
-        if type is KeyboardInterrupt:
-            self.do_quit_onboard()
-        else:
-            sys.__excepthook__(type, value, traceback)
+        # Additional instances after the first one don't open main windows.
+        # Make sure that startup is announced as complete or unity will
+        # block the launcher icon for 3 seconds.
+        Gdk.notify_startup_complete()
+
+        # Shut up error messages on SIGTERM in lightdm:
+        # "sys.excepthook is missing, lost sys.stderr"
+        # See http://bugs.python.org/issue11380 for more.
+        # Python 2.7, Precise
+        try:
+            sys.stdout.close()
+        except:
+            pass
+        try:
+            sys.stderr.close()
+        except:
+            pass
 
     def do_activate(self):
         """
         App instance entry point.
+        This is always called in the context of the first instance.
         """
         if len(self.get_windows()) == 0:
             self.init()
             self.add_window(self._window)
         else:
             if self.keyboard:
-                self.keyboard.set_user_visible(True)
+                self.keyboard.set_visible(True)
 
     def init(self):
         self.keyboard_state = None
@@ -99,8 +132,19 @@ class OnboardGtk(Gtk.Application):
         # finish config initialization
         config.init()
 
+        # Release pressed keys when onboard is killed.
+        # Don't keep enter key stuck when being killed from lightdm.
+        self._osk_util = osk.Util()
+        self._osk_util.set_unix_signal_handler(signal.SIGTERM, self.on_sigterm)
+        self._osk_util.set_unix_signal_handler(signal.SIGINT, self.on_sigint)
+
         sys.path.append(os.path.join(config.install_dir, 'scripts'))
 
+        # Create the keyboard
+        # Care for toolkit independency only once there is another
+        # supported one besides GTK.
+        self.keyboard = KeyboardGTK()
+
         # load the initial layout
         _logger.info("Loading initial layout")
         self.reload_layout()
@@ -117,6 +161,10 @@ class OnboardGtk(Gtk.Application):
             self.do_connect(self._window, "quit-onboard",
                             lambda x: self.do_quit_onboard())
 
+        if not config.xid_mode:  # don't flash the icon palette in lightdm
+            icp = IconPalette()
+            icp.connect("activated", self._on_icon_palette_acticated)
+            self._window.icp = icp
         self._window.application = self
         self._window.set_keyboard(self.keyboard)
 
@@ -134,6 +182,9 @@ class OnboardGtk(Gtk.Application):
             if not options.y is None:
                 rect.y = options.y
 
+            # Make sure the keyboard fits on screen
+            rect = self._window.limit_size(rect)
+
             if rect != self._window.get_rect():
                 orientation = self._window.get_screen_orientation()
                 self._window.write_window_rect(orientation, rect)
@@ -142,6 +193,11 @@ class OnboardGtk(Gtk.Application):
         # show/hide the window
         self.keyboard.set_startup_visibility()
 
+        # keep keyboard window and icon palette on top of dash
+        if not config.xid_mode: # be defensive, not necessary when embedding
+            self._osk_util.keep_windows_on_top([self._window,
+                                                self._window.icp])
+
         # connect notifications for keyboard map and group changes
         self.keymap = Gdk.Keymap.get_default()
         self.do_connect(self.keymap, "keys-changed", self.cb_keys_changed) # map changes
@@ -280,16 +336,24 @@ class OnboardGtk(Gtk.Application):
             not config.check_gnome_accessibility(self._window):
             config.auto_show.enabled = False
 
+    def on_sigterm(self):
+        """
+        Exit onboard on kill.
+        """
+        _logger.debug("SIGTERM received")
+        self.do_quit_onboard()
+
+    def on_sigint(self):
+        """
+        Exit onboard on Ctrl+C press.
+        """
+        _logger.debug("SIGINT received")
+        self.do_quit_onboard()
+
     def do_connect(self, instance, signal, handler):
         handler_id = instance.connect(signal, handler)
         self._connections.append((instance, handler_id))
 
-    def on_signal(self, signum, frame):
-        if signum == signal.SIGTERM:
-            _logger.debug("SIGTERM received")
-            self.cleanup()
-            sys.exit(1)
-
     # Method concerning the taskbar
     def show_hide_taskbar(self):
         """
@@ -298,16 +362,13 @@ class OnboardGtk(Gtk.Application):
         This method should be called every time such an alternative way
         is activated or deactivated.
         """
-        # Always skip taskbar for now (Precise)
-        # Otherwise unity and unity-2d won't stop wiggling onboards icon.
-        if config.allow_iconifying and \
-           not config.has_unhide_option():
-            self._window.set_property('skip-taskbar-hint', False)
-        else:
-            self._window.set_property('skip-taskbar-hint', True)
-
+        if self._window:
+            self._window.update_taskbar_hint()
 
     # Method concerning the icon palette
+    def _on_icon_palette_acticated(self, widget):
+        self.keyboard.toggle_visible()
+
     def cb_icp_in_use_toggled(self, icp_in_use):
         """
         This is the callback that gets executed when the user toggles
@@ -447,34 +508,23 @@ class OnboardGtk(Gtk.Application):
         _logger.info("Loading keyboard layout from " + layout_filename)
         if (color_scheme_filename):
             _logger.info("Loading color scheme from " + color_scheme_filename)
-        if self.keyboard:
-            self.keyboard.cleanup()
-
-        # Fix for LP: #897678, onBoard shifts after changing Language Layout
-        # The idea is to no longer recreate the keyboard widget and
-        # instead just update its contents. This solves a couple of
-        # weird positioning bugs when the keyboard widget changes while
-        # the main window is hidden.
-        # This is messy but works for now. Trunk will have to replace
-        # KeyboardSVG with proper widget-independent loading code.
+        
         vk = self.get_vk()
-        keyboard = KeyboardSVG(vk, layout_filename,
-                                    color_scheme_filename)
-        if not self.keyboard:
-            self.keyboard = keyboard
-        layout, color_scheme = keyboard.layout, keyboard.color_scheme
-        keyboard.cleanup()
 
-        if self.keyboard and keyboard:
-            self.keyboard.vk = vk
-            self.keyboard.layout = layout
-            self.keyboard.color_scheme = color_scheme
-            self.keyboard.initial_update()
-            self.keyboard.update_ui()
-            self.keyboard.redraw()
+        color_scheme = ColorScheme.load(color_scheme_filename) \
+                       if color_scheme_filename else None
+        layout = LayoutLoaderSVG().load(vk, layout_filename, color_scheme)
 
-            if self._window and self._window.icp:
-                self._window.icp.queue_draw()
+        self.keyboard.cleanup()
+        self.keyboard.vk = vk
+        self.keyboard.layout = layout
+        self.keyboard.color_scheme = color_scheme
+        self.keyboard.initial_update()
+        self.keyboard.update_ui()
+        self.keyboard.redraw()
+
+        if self._window and self._window.icp:
+            self._window.icp.queue_draw()
 
     def get_vk(self):
         if not self._vk:
@@ -485,7 +535,7 @@ class OnboardGtk(Gtk.Application):
             except virtkey.error as e:
                 t = time.time()
                 if t > self._vk_error_time + .2: # rate limit to once per 200ms
-                    _logger.warning("vk: "+str(e))
+                    _logger.warning("vk: " + unicode_str(e))
                     self._vk_error_time = t
 
         return self._vk
@@ -524,6 +574,7 @@ class OnboardGtk(Gtk.Application):
     def final_cleanup(self):
         config.final_cleanup()
 
+
 def cb_any_event(event, onboard):
     # Update layout on keyboard group changes
     # XkbStateNotify maps to Gdk.EventType.NOTHING
@@ -543,6 +594,10 @@ def cb_any_event(event, onboard):
             a += [event.state]
         if type == Gdk.EventType.CONFIGURE:
             a += [event.x, event.y, event.width, event.height]
+        if type == Gdk.EventType.WINDOW_STATE:
+            a += [event.window_state]
+        if type == Gdk.EventType.UNMAP:
+            a += [event.window, "0x{:x}".format(event.window.get_xid())]
         print(*a)
 
     if type == Gdk.EventType.NOTHING:
