@@ -26,9 +26,11 @@
 #if PY_MAJOR_VERSION >= 3
     #define PyString_FromString PyUnicode_FromString
     #define PyString_FromStringAndSize PyUnicode_FromStringAndSize
+    #define PyString_AsString PyByteArray_AsString
 
     #define PyInt_FromLong PyLong_FromLong
     #define PyInt_AsLong PyLong_AsLong
+    #define PyInt_Check PyLong_Check
 #else
 #endif
 
