@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 # -*- coding: utf-8 -*-
 """ Onboard preferences utility """
 
@@ -12,24 +12,23 @@ from subprocess import Popen
 from xml.parsers.expat import ExpatError
 from xml.dom import minidom
 import gettext
+from dbus.mainloop.glib import DBusGMainLoop
 
 from gi.repository import GObject, Pango, Gdk, Gtk
 
-# setup gettext, install _() function for all modules
-app = "onboard"
-gettext.install(app, unicode=True)
-
-from Onboard.KeyboardSVG import KeyboardSVG
-from Onboard.SnippetView import SnippetView
-from Onboard.Appearance  import Theme, ColorScheme
-from Onboard.Scanner     import ScanMode, ScanDevice
-from Onboard.utils       import show_ask_string_dialog, \
-                                show_confirmation_dialog
+# install translation function _() for all modules
+from Onboard.LayoutLoaderSVG import LayoutLoaderSVG
+from Onboard.SnippetView     import SnippetView
+from Onboard.Appearance      import Theme, ColorScheme
+from Onboard.Scanner         import ScanMode, ScanDevice
+from Onboard.utils           import show_ask_string_dialog, \
+                                    show_confirmation_dialog, \
+                                    unicode_str
 
 from virtkey import virtkey
-from osk     import Devices
-
+from Onboard.osk import Devices
 
+app = "onboard"
 
 ### Logging ###
 import logging
@@ -57,6 +56,9 @@ def format_list_item(text, issystem):
 class Settings:
     def __init__(self,mainwin):
 
+        # Use D-bus main loop by default
+        DBusGMainLoop(set_as_default=True)
+
         # finish config initialization
         config.init()
 
@@ -378,7 +380,7 @@ class Settings:
         if new_layout_name:
             new_filename = os.path.join(self.user_layout_root, new_layout_name) + \
                            config.LAYOUT_FILE_EXTENSION
-            KeyboardSVG.copy_layout(config.layout_filename, new_filename)
+            LayoutLoaderSVG.copy_layout(config.layout_filename, new_filename)
             self.update_layoutList()
             self.open_user_layout_dir()
 
@@ -400,8 +402,8 @@ class Settings:
         try:
             Popen([filename, "universal-access"])
         except OSError as e:
-            _logger.warning(_("System settings not found"
-                              " ({}): {}").format(filename, str(e)))
+            _logger.warning(_format("System settings not found ({}): {}",
+                                    filename, unicode_str(e)))
 
     def update_num_resize_handles_combobox(self):
         self.num_resize_handles_list = Gtk.ListStore(str, int)
@@ -475,7 +477,7 @@ class Settings:
         if response == Gtk.ResponseType.OK:
             filename = chooser.get_filename()
 
-            f = open(filename)
+            f = open(filename, encoding="UTF-8")
             sokdoc = minidom.parse(f).documentElement
             for p in sokdoc.getElementsByTagName("pane"):
                 fn = p.attributes['filename'].value
@@ -494,7 +496,7 @@ class Settings:
         if sel:
             filename = self.layoutList.get_value(sel.get_selected()[1], 1)
 
-            KeyboardSVG.remove_layout(filename)
+            LayoutLoaderSVG.remove_layout(filename)
 
             config.layout_filename = self.layoutList[0][1] \
                                      if len(self.layoutList) else ""
@@ -506,7 +508,7 @@ class Settings:
 
         layouts = []
         for filename in filenames:
-            file_object = open(filename)
+            file_object = open(filename, encoding="UTF-8")
             try:
                 sokdoc = minidom.parse(file_object).documentElement
 
@@ -573,9 +575,9 @@ class Settings:
             if not os.path.exists(new_filename):
                 break
 
-            question = _("This theme file already exists.\n'{filename}'" \
-                         "\n\nOverwrite it?") \
-                        .format(filename=new_filename)
+            question = _format("This theme file already exists.\n'{filename}'"
+                               "\n\nOverwrite it?",
+                               filename=new_filename)
             if show_confirmation_dialog(question, self.window):
                 break
 
@@ -743,7 +745,6 @@ class Settings:
         return None
 
 
-
 class ThemeDialog:
     """ Customize theme dialog """
 
@@ -764,8 +765,10 @@ class ThemeDialog:
         self.color_scheme_combobox = builder.get_object("color_scheme_combobox")
         self.font_combobox = builder.get_object("font_combobox")
         self.font_attributes_view = builder.get_object("font_attributes_view")
-        self.roundrect_radius_scale = builder.get_object(
-                                               "roundrect_radius_scale")
+        self.background_gradient_scale = builder.get_object(
+                                               "background_gradient_scale")
+        self.key_roundness_scale = builder.get_object(
+                                               "key_roundness_scale")
         self.key_size_scale = builder.get_object(
                                                "key_size_scale")
         self.gradients_box = builder.get_object("gradients_box")
@@ -775,6 +778,10 @@ class ThemeDialog:
                                                "key_stroke_gradient_scale")
         self.key_gradient_direction_scale = builder.get_object(
                                                "key_gradient_direction_scale")
+        self.key_shadow_strength_scale = builder.get_object(
+                                               "key_shadow_strength_scale")
+        self.key_shadow_size_scale = builder.get_object(
+                                               "key_shadow_size_scale")
         self.revert_button = builder.get_object("revert_button")
         self.superkey_label_combobox = builder.get_object(
                                                "superkey_label_combobox")
@@ -820,13 +827,18 @@ class ThemeDialog:
         self.update_color_schemeList()
         self.update_fontList()
         self.update_font_attributesList()
-        self.roundrect_radius_scale.set_value(self.theme.roundrect_radius)
+        self.background_gradient_scale.set_value(self.theme.background_gradient)
+        self.key_roundness_scale.set_value(self.theme.roundrect_radius)
         self.key_size_scale.set_value(self.theme.key_size)
         self.key_fill_gradient_scale.set_value(self.theme.key_fill_gradient)
         self.key_stroke_gradient_scale. \
                 set_value(self.theme.key_stroke_gradient)
         self.key_gradient_direction_scale. \
                 set_value(self.theme.key_gradient_direction)
+        self.key_shadow_strength_scale. \
+                set_value(self.theme.key_shadow_strength)
+        self.key_shadow_size_scale. \
+                set_value(self.theme.key_shadow_size)
         self.update_superkey_labelList()
         self.superkey_label_size_checkbutton. \
                 set_active(bool(self.theme.get_superkey_size_group()))
@@ -917,7 +929,7 @@ class ThemeDialog:
                 self.font_combobox.set_active_iter(it)
 
     def font_combobox_row_separator_func(self, model, iter, data):
-        return model.get_value(iter, 0) == "-"
+        return unicode_str(model.get_value(iter, 0)) == "-"
 
     def update_font_attributesList(self):
         treeview = self.font_attributes_view
@@ -976,8 +988,11 @@ class ThemeDialog:
 
         self.superkey_label_combobox.set_model(self.superkey_label_model)
 
-    def on_theme_notebook_switch_page(self, widget, gpage, page_num):
-        ThemeDialog.current_page = page_num
+    def on_background_gradient_value_changed(self, widget):
+        value = float(widget.get_value())
+        config.theme_settings.background_gradient = value
+        self.theme.background_gradient = value
+        self.update_sensivity()
 
     def on_key_style_combobox_changed(self, widget):
         value = self.key_style_list.get_value( \
@@ -986,7 +1001,7 @@ class ThemeDialog:
         config.theme_settings.key_style = value
         self.update_sensivity()
 
-    def on_roundrect_value_changed(self, widget):
+    def on_key_roundness_value_changed(self, widget):
         radius = int(widget.get_value())
         config.theme_settings.roundrect_radius = radius
         self.theme.roundrect_radius = radius
@@ -1023,6 +1038,18 @@ class ThemeDialog:
         self.theme.key_gradient_direction = value
         self.update_sensivity()
 
+    def on_key_shadow_strength_value_changed(self, widget):
+        value = float(widget.get_value())
+        config.theme_settings.key_shadow_strength = value
+        self.theme.key_shadow_strength = value
+        self.update_sensivity()
+
+    def on_key_shadow_size_value_changed(self, widget):
+        value = float(widget.get_value())
+        config.theme_settings.key_shadow_size = value
+        self.theme.key_shadow_size = value
+        self.update_sensivity()
+
     def on_font_combobox_changed(self, widget):
         if not self.in_update:
             self.store_key_label_font()
@@ -1061,6 +1088,9 @@ class ThemeDialog:
         self.theme.set_superkey_label(label, size_group)
         config.theme_settings.key_label_overrides = self.theme.key_label_overrides
 
+    def on_theme_notebook_switch_page(self, widget, gpage, page_num):
+        ThemeDialog.current_page = page_num
+
 
 class ScannerDialog(object):
     """ Scanner settings dialog """
@@ -1303,7 +1333,7 @@ class ScannerDialog(object):
         action = model.get_value(it, self.COL_ACTION)
         dev_map[value] = action
 
-        for k, v in dev_map.iteritems():
+        for k, v in dev_map.items():
             if k != value and v == action:
                 del dev_map[k]
                 break
@@ -1342,7 +1372,7 @@ class ScannerDialog(object):
             self.update_input_devices()
 
     def get_value_for_action(self, action, dev_map):
-        for k, v in dev_map.iteritems():
+        for k, v in dev_map.items():
             if v == action:
                 return k
 
@@ -1372,19 +1402,19 @@ class CellRendererMapping(Gtk.CellRendererText):
     Custom cell renderer that displays device buttons as labels.
     """
 
-    __gproperties__ = { b'button'      : (GObject.TYPE_INT,
-                                          '', '', 0, MAX_GINT32, 0,
-                                          GObject.PARAM_READWRITE),
-                        b'key'         : (GObject.TYPE_INT,
-                                          '', '', 0, MAX_GINT32, 0,
-                                          GObject.PARAM_READWRITE),
-                        b'pointer-mode': (bool, '', '', True,
-                                          GObject.PARAM_READWRITE) }
+    __gproperties__ = { str('button')      : (GObject.TYPE_INT,
+                                             '', '', 0, MAX_GINT32, 0,
+                                             GObject.PARAM_READWRITE),
+                        str('key')         : (GObject.TYPE_INT,
+                                             '', '', 0, MAX_GINT32, 0,
+                                             GObject.PARAM_READWRITE),
+                        str('pointer-mode') : (bool, '', '', True,
+                                             GObject.PARAM_READWRITE) }
 
-    __gsignals__ = { b'mapping-edited' : (GObject.SIGNAL_RUN_LAST,
-                                          None, (str, int, bool)),
-                     b'mapping-cleared': (GObject.SIGNAL_RUN_LAST,
-                                          None, (str, bool)) }
+    __gsignals__ = { str('mapping-edited') : (GObject.SIGNAL_RUN_LAST,
+                                             None, (str, int, bool)),
+                     str('mapping-cleared'): (GObject.SIGNAL_RUN_LAST,
+                                             None, (str, bool)) }
 
     def __init__(self):
         super(CellRendererMapping, self).__init__(editable=True)
@@ -1554,8 +1584,8 @@ class EditableBox(Gtk.EventBox, Gtk.CellEditable):
     Container that implements the Gtk.CellEditable interface.
     """
 
-    __gproperties__ = { b'editing-canceled': (bool, '', '', False,
-                                              GObject.PARAM_READWRITE) }
+    __gproperties__ = { str('editing-canceled'): (bool, '', '', False,
+                                                  GObject.PARAM_READWRITE) }
 
     def __init__(self, child=None):
         super(EditableBox, self).__init__()
