@@ -8,8 +8,9 @@ import time
 import re
 import traceback
 import colorsys
+import gettext
 from subprocess import Popen
-from math import pi, sqrt, sin
+from math import pi, sin, cos, exp, sqrt, log
 from contextlib import contextmanager
 
 from gi.repository import GObject, Gtk
@@ -607,6 +608,13 @@ def brighten(amount, r, g, b, a=0.0):
         l = 0.0
     return list(colorsys.hls_to_rgb(h, l, s)) + [a]
 
+def linint_rgba(k, rgba1, rgba2):
+    """ interpolate between two colors """
+    linint = lambda k, a, b: a + (b - a) * k
+    return [linint(k, rgba1[0], rgba12[0]),
+            linint(k, rgba1[1], rgba12[1]),
+            linint(k, rgba1[2], rgba12[2]),
+            linint(k, rgba1[3], rgba12[3])]
 
 def roundrect_arc(context, rect, r = 15):
     x0,y0 = rect.x, rect.y
@@ -694,6 +702,45 @@ def round_corners(cr, r, x, y, w, h):
     cr.close_path()
     cr.fill()
 
+def gradient_line(rect, alpha):
+    # Find rotated gradient start and end points.
+    # Line end points follow the largest extent of the rotated rectangle.
+    # The gradient reaches across the entire rectangle.
+    x0, y0, w, h = rect.x, rect.y, rect.w, rect.h
+    a = w / 2.0
+    b = h / 2.0
+    coords = [(-a, -b), (a, -b), (a, b), (-a, b)]
+    vx = [c[0]*cos(alpha)-c[1]*sin(alpha) for c in coords]
+    dx = max(vx) - min(vx)
+    r = dx / 2.0
+    return (r * cos(alpha) + x0 + a,
+            r * sin(alpha) + y0 + b,
+           -r * cos(alpha) + x0 + a,
+           -r * sin(alpha) + y0 + b)
+
+def drop_shadow(cr, pattern, bounds, blur_radius = 4.0, offset = (0, 0),
+                                  alpha=0.06, steps=4):
+    """
+    Mostly works, but has issues with clipping artefacts for
+    damage rects smaller than the full window rect.
+    """
+    origin = bounds.get_center()
+    for i in range(steps):
+
+        x = (i if i else 0.5) / float(steps)
+        k = sqrt(abs(log(1-x))) * 0.7 * blur_radius # gaussian
+        #k = i / float(steps) * blur_radius         # linear
+
+        x_scale = (bounds.w + k) / bounds.w
+        y_scale = (bounds.h + k) / bounds.h
+        cr.save()
+        cr.translate(*origin)
+        cr.scale(x_scale, y_scale)
+        cr.translate(-origin[0] + offset[0], -origin[1] + offset[1])
+        cr.set_source_rgba(0.0, 0.0, 0.0, alpha)
+
+        cr.mask(pattern)
+        cr.restore()
 
 @contextmanager
 def timeit(s, out=sys.stdout):
@@ -793,13 +840,12 @@ class DelayedLauncher(Timer):
         self.start(delay)
 
     def on_timer(self):
-        _logger.debug(_("launching '{}'") \
-                        .format(" ".join(self.args)))
+        _logger.debug("launching '{}'".format(" ".join(self.args)))
         try:
             Popen(self.args)
         except OSError as e:
-            _logger.warning(_("Failed to execute '{}', {}") \
-                            .format(" ".join(self.args), e))
+            _logger.warning(_format("Failed to execute '{}', {}", \
+                            " ".join(self.args), e))
         return False
 
 
@@ -807,7 +853,7 @@ class FadeTimer(Timer):
     """
     Sine-interpolated fade between two values, e.g. opacities.
     """
-
+    value = None
     start_value = None
     target_value = None
     iteration = 0   # just a counter of on_timer calls since start
@@ -819,6 +865,7 @@ class FadeTimer(Timer):
         Start value fade.
         duration: fade time in seconds, 0 for immediate value change
         """
+        self.value = start_value
         self.start_value = start_value
         self._start_time = time.time()
         self._duration = duration
@@ -838,22 +885,32 @@ class FadeTimer(Timer):
         Timer.stop(self)
 
     def on_timer(self):
-        elapsed = time.time() - self._start_time
-        if self._duration:
-            lin_progress = min(1.0, elapsed / self._duration)
-        else:
-            lin_progress = 1.0
-        sin_progress = (sin(lin_progress * pi - pi / 2.0) + 1.0) / 2.0
-        self.value = sin_progress * (self.target_value - self.start_value) + \
-                  self.start_value
+        self.value, done = Fade.sin_fade(self._start_time, self._duration,
+                                         self.start_value, self.target_value)
 
-        done = lin_progress >= 1.0
         if self._callback:
             self._callback(self.value, done, *self._callback_args)
 
         self.iteration += 1
         return not done
 
+class Fade:
+    """ Helper for opacity fading """
+    @staticmethod
+    def sin_fade(start_time, duration, start_value, target_value):
+        elapsed = time.time() - start_time
+        if duration:
+            lin_progress = min(1.0, elapsed / duration)
+        else:
+            lin_progress = 1.0
+        return(Fade.sin_int(lin_progress, start_value, target_value),
+               lin_progress >= 1.0)
+
+    @staticmethod
+    def sin_int(lin_progress, start_value, target_value):
+        sin_progress = (sin(lin_progress * pi - pi / 2.0) + 1.0) / 2.0
+        return sin_progress * (target_value - start_value) + start_value
+
 
 class TreeItem(object):
     """
@@ -938,9 +995,9 @@ class Version(object):
         major = 0
         minor = 0
         try:
-            if components >= 1:
+            if len(components) >= 1:
                 major = int(components[0])
-            if components >= 2:
+            if len(components) >= 2:
                 minor = int(components[1])
         except ValueError:
             pass
@@ -950,7 +1007,14 @@ class Version(object):
     def to_string(self):
         return "{major}.{minor}".format(major=self.major, minor=self.minor)
 
-    def __cmp__(self, other):
+    def __eq__(self, other): return self._cmp(other) == 0
+    def __ne__(self, other): return self._cmp(other) != 0
+    def __lt__(self, other): return self._cmp(other) < 0
+    def __le__(self, other): return self._cmp(other) <= 0
+    def __gt__(self, other): return self._cmp(other) > 0
+    def __ge__(self, other): return self._cmp(other) >= 0
+
+    def _cmp(self, other):
         if self.major < other.major:
             return -1
         if self.major > other.major:
@@ -962,3 +1026,88 @@ class Version(object):
         return 0
 
 
+class Process:
+    """ Process utilities """
+
+    @staticmethod
+    def get_cmdline(pid):
+        """ Returns the command line for process id pid """
+        cmdline = ""
+        with open("/proc/%s/cmdline" % pid) as f:
+            cmdline = f.read()
+        return cmdline
+
+    @staticmethod
+    def was_launched_by(process_name):
+        """ Checks if this process was launched by <process_name> """
+        ppid = os.getppid()
+        if ppid:
+            cmdline = Process.get_cmdline(ppid)
+            return process_name in cmdline
+        return False
+
+def unicode_str(obj, encoding = "utf-8"):
+    """
+    Safe str() function that always returns an unicode string.
+    Do nothing if the string was already unicode.
+    """
+    if sys.version_info.major >= 3:  # python 3?
+        return str(obj)
+
+    if type(obj) == unicode:         # unicode string?
+        return obj
+
+    if hasattr(obj, "__unicode__"):  # Exception object?
+        return unicode(obj)
+
+    return str(obj).decode("utf-8")  # strings, numbers, ...
+
+
+class Translation:
+    """
+    Translations occasionally contain errors in format fields that
+    prevent onboard from starting up. This class aims to catch these
+    errors gracefully, report and ignore bad translations and then
+    just go on.
+
+    Common errors have been:
+        - bad field names, e.g. "{filename}" was translated "{path}"
+        - bad anonymous fields, e.g. "{}" was translated "{ }"
+    """
+    @staticmethod
+    def install(domain):
+        """ setup gettext, install _() function for all modules """
+
+        try:
+            import builtins
+        except ImportError:
+            builtins = sys.modules["__builtin__"]  # python 2.x
+
+        t = Translation()
+        t.translation = gettext.translation(domain, fallback=True)
+
+        builtins.__dict__['_'] = t.ugettext
+        builtins.__dict__['_format'] = t.format
+
+    def ugettext(self, msgid):
+        if sys.version_info.major < 3:  # python 2?
+            return self.translation.ugettext(msgid)
+        return self.translation.gettext(msgid)
+
+    def format(self, msgid, *args, **kwargs):
+        """ Safe replacement for str.format() """
+
+        msgstr = self.ugettext(msgid)
+        try:
+            result = msgstr.format(*args, **kwargs)
+        except (KeyError, IndexError, UnicodeDecodeError) as e:
+            result = msgid.format(*args, **kwargs)
+
+            _logger.warning("_format: Skipping bad translation "
+                            "msgid='{}' msgstr='{}' {}: {}" \
+                            .format(msgid, msgstr,
+                                    e.__class__.__name__,
+                                    unicode_str(e)))
+        return result
+
+
